package com.usky.dxtop.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.usky.dxtop.common.constant.UserConstants;
import com.usky.dxtop.common.core.page.CommonPage;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.common.utils.StringUtils;
import com.usky.dxtop.common.utils.file.FileUtils;
import com.usky.dxtop.mapper.QrCodeMapper;
import com.usky.dxtop.model.QrCode;
import com.usky.dxtop.model.TopChannel;
import com.usky.dxtop.model.TopRadio;
import com.usky.dxtop.service.QrCodeService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author yq
 * @since 2021-08-26
 */
@Service
public class QrCodeServiceImpl extends ServiceImpl<QrCodeMapper, QrCode> implements QrCodeService {

    @Override
    public String checkNameUnique(QrCode qrCode) {
        Long configId = StringUtils.isNull(qrCode.getId()) ? -1L : qrCode.getId();
        LambdaQueryWrapper<QrCode> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(QrCode::getName,qrCode.getName())
                .eq(QrCode::isDelFlag,0);
        QrCode one = this.getOne(queryWrapper);
        if (StringUtils.isNotNull(one) && one.getId() != configId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public CommonPage<QrCode> list(Integer current,Integer size,String name) {
        IPage<QrCode> page = new Page<>(current, size);
        page = baseMapper.list(page,name);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean remove(Long id) {
        QrCode qrCode = new QrCode();
        qrCode.setId(id);
        qrCode.setDelFlag(true);
        boolean b = this.updateById(qrCode);
        if (b){
            qrCode = this.getById(id);
            b = FileUtils.deleteFile(qrCode.getQrPath());
            if (!b){
                throw new CustomException("删除二维码文件失败");
            }
        }
        return b;
    }
}
